"""

"""

CREATE_EDGES_SAMPLES = {
    "has_satellite": {
        "label": "has_satellite",
        "properties": {
            "x_prop": "xyz",
        }
    },
    "has_animals": {
        "label": "has_animals",
        "properties": {
            "x_prop": "xyz",
        }
    }
}
